from functools import partial


class AbstractDbProvider(object):
    def connect(self):
        return NotImplementedError(self.__class__.__name__)

    def execute(self, *args, **kwargs):
        return NotImplementedError(self.__class__.__name__)

    def close(self):
        return NotImplementedError(self.__class__.__name__)


class BaseDbProvider(AbstractDbProvider):
    def __init__(self):
        self.query_storage = None

    def __getattr__(self, item):
        if self.query_storage:
            query_method = getattr(self.query_storage, item)
            if query_method:
                return partial(self._make_execute, query_method)
            else:
                raise AttributeError('Не найден метод запроса {} в хранилище запросов {}'.format(
                    item, self.query_storage))
        else:
            raise TypeError('Не определен класс хранилища запросов. Добавить хранилище запросов \n'
                            'context.нужный_БД_провайдер.query_storage = Класс_хранилище_запросов. \n'
                            'Или альтернативный способ выполнения запросов через метод execute: \n'
                            'context.нужный_БД_провайдер.execute(запрос)')

    def _make_execute(self, query, *args, **kwargs):
        """
        Собрать запрос и выполнить стандартным методом execute
        :param query: метод запроса
        :param args: параметры запроса
        :param kwargs: параметры запроса
        :return: список строк из БД
        """
        params = kwargs.get('params', None)
        if params:
            del kwargs['params']
        q = query(*args, **kwargs)
        if isinstance(q, tuple):
            return self.execute(*q, params=params)
        else:
            return self.execute(q, params=params)
