import pyodbc

from pb_common.db import BaseDbProvider
from pb_common.utils import support
from pb_common.utils.logger import get_logger

logger = get_logger(__name__)


class MSSQLProvider(BaseDbProvider):
    """
    Класс работы с MS SQL.
    """

    def __init__(self, host, port, name, login, password):
        # параметры аутетификации к БД
        super().__init__()
        self._host = host
        self._port = port
        self._name = name
        self._login = login
        self._password = password

        # параметры для работы внутри БД
        self.conn = None
        self.cursor = None

    @property
    def get_cursor(self):
        if not self.cursor or not self.conn:
            self.connect()
        return self.cursor

    def __rows_to_dict_list(self, result):
        if result:
            columns = [i[0].upper() for i in self.get_cursor.description]
            return [
                dict(list(zip(columns, [x if isinstance(x, str) else x for x in row]))) for row in result
            ]
        else:
            return []

    def connect(self):
        try:
            self.conn = pyodbc.connect(
                "DRIVER={ODBC Driver 17 for SQL Server};SERVER=%s;DATABASE=%s;UID=%s;PWD=%s" %
                (self._host, self._name, self._login, self._password)
            )
        except Exception as e:
            logger.error(e)
            self.conn = pyodbc.connect(r'Driver={SQL Server};Server=%s;Database=%s;Trusted_Connection=yes;' %
                                         (self._host, self._name))
        assert self.conn, "Failed connection to mssql!"
        logger.info("Connected to MSSQL")
        self.cursor = self.conn.cursor()
        logger.info("Get MSSQL cursor.")
        return self

    def execute(self, req, params=None):
        support.validate_type(params, None, list, tuple)
        try:
            logger.info('Execute query: {}'.format(req))
            self.get_cursor.execute(req, params)
            result = self.get_cursor.fetchall()
            return self.__rows_to_dict_list(result)
        except pyodbc.ProgrammingError:
            self.conn.commit()
            return None
        except Exception as e:
            logger.error(e)
            raise e

    def close(self):
        try:
            self.cursor.close()
            self.conn.close()
        except Exception as e:
            logger.error(e)
        finally:
            self.conn = None
            self.cursor = None
        logger.info("Closing opened ms sql cursor.")
