import os
import uuid
from functools import partial

import requests
import xml.dom.minidom




from urllib3.exceptions import InsecureRequestWarning

from pb_common.services import AbstractRequestProvider
from pb_common.utils.logger import get_logger
from pb_common.utils import support

__author__ = "Maksim Senchuk"


logger = get_logger(__name__)


requests.urllib3.disable_warnings(InsecureRequestWarning)
requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS = 'RSA+AESGCM:RSA+AES:RSA+HIGH:RSA+3DES:!aNULL:!eNULL:!MD5'


class FimiMethodUndefined(Exception):
    pass


class FimiProvider(AbstractRequestProvider):
    def __init__(self, service: str, protocol: str, host: str, port: int, methods: dict,
                 path=None, headers=None, cert=None):
        self.service_name = service
        self._protocol = protocol
        self._host = host
        self._port = port
        self._methods = methods
        self.cert = cert
        self.cert_key = None
        self._path = path
        self._headers = dict(headers) if headers else {}

    def base_url(self):
        url = "{protocol}://{host}:{port}".format(protocol=self._protocol, host=self._host, port=self._port)
        if self._path:
            url += self._path if self._path.startswith('/') else '/{}'.format(self._path)
        return url

    def __getattr__(self, item):
        if dict(self._methods).get(item):
            http_method = self._methods[item][0]
            method = self._methods[item][1]
            url = self.base_url()
            _headers = {'Content-Type': self._headers['Content-Type'].format(method=method)}
            return partial(self.send, method=http_method, url=url, headers=_headers)
        else:
            raise FimiMethodUndefined("Не найден {} метод {} сервиса в списке разрешенных методов: {}".format(
                item, self.service_name, self._methods))

    @property
    def ssl_cert(self):
        if not self.cert_key:
            cert_, key_ = None, None
            for root, dirs, files in os.walk(os.getcwd()):
                if self.cert.file in files:
                    cert_ = os.path.join(root, self.cert.file)
                if self.cert.key in files:
                    key_ = os.path.join(root, self.cert.key)
                if cert_ and key_:
                    self.cert_key = (cert_, key_)
                    break
        return self.cert_key

    def send(self, method, url, **kwargs):
        request_uuid = str(uuid.uuid4())
        if kwargs.get('headers'):
            kwargs['headers'].update({'x-uuid': request_uuid})
        else:
            kwargs.update(dict(headers={'x-uuid': request_uuid}))
        logger.info("Send to '{}' request: {} {}".format(self.service_name, method, url))
        logger.info("Request params: {}".format(kwargs))
        r = requests.request(method=method, url=url, cert=self.ssl_cert, verify=False, timeout=60.0, **kwargs)
        _data = support.datetime_parser(support.xml_formatter(r.text))
        fault = support.find_item(_data, 'Fault')
        fault = fault if fault else {}
        data = support.find_item(_data, 'Response')
        data = data if data else {}
        setattr(r, 'data', data)
        setattr(r, 'fault', fault)
        dom = xml.dom.minidom.parseString(r.text)
        text = dom.toprettyxml()
        logger.info("Response: {} {}\n{}\n{}".format(r.status_code, r.reason, r.headers, text))
        return r
