import paramiko

from pb_common.services import AbstractSSHProvider
from pb_common.utils.logger import get_logger

logger = get_logger(__name__)


class ShellExecuteError(Exception):
    pass


class SSHProvider(AbstractSSHProvider):
    def __init__(self, host: str, port: int, login: str, password: str):
        self._host = host
        self._port = port
        self._login = login
        self._password = password
        self._client = None

    @property
    def get_client(self):
        if not self._client:
            self.connect()
        return self._client

    def connect(self):
        try:
            self._client = paramiko.SSHClient()
            type_host_key = paramiko.AutoAddPolicy()
            self._client.set_missing_host_key_policy(type_host_key)
            self._client.connect(
                hostname=self._host,
                username=self._login,
                password=self._password,
                port=self._port
            )
            logger.info("Open SSH connection to host {}".format(self._host))
        except Exception as e:
            logger.error(e)
            raise e
        return self

    def close(self):
        try:
            self._client.close()
        except Exception as e:
            logger.error(e)
        logger.info("Closed SSH connection to host {}".format(self._host))

    def execute(self, command):
        try:
            logger.info("Execute: %s" % command)
            stdin, stdout, stderr = self.get_client.exec_command("""%s""" % command)
            data = dict(stdout=stdout.read(), stderr=stderr.read())
            if data["stderr"]:
                logger.info("Return. Output: %s" % str(stderr))
                raise ShellExecuteError("Return error: %s" % str(stderr))
            logger.info("Execute success. Output: %s" % str(data))
            return data
        except Exception as e:
            logger.error(e)
            raise e
