# -*- coding: utf-8 -*-
import os
import logging.handlers
import logging
import sys

from pb_common.utils import support


class Logger(object):
    def __init__(self, format_log='%(asctime)s | %(levelname)s | %(name)s:%(lineno)d | %(message)s', level_log='INFO',
                 log_file='console.log'):
        logging.disable(logging.NOTSET)
        folder_log = os.path.join(support.root_dir(), "log")
        self.log_file = os.path.join(folder_log, log_file)

        if not os.path.isdir(folder_log):
            os.mkdir(folder_log)

        self.log_level = logging.getLevelName(os.environ.get('LOG_LEVEL', level_log).upper().strip())
        self.formatter = logging.Formatter(format_log)

        self._stream_handler = self.stream_handler(self.log_level, self.formatter)
        self._file_handler = self.rotating_file_handler(self.log_file, self.log_level, self.formatter)

        _root = logging.getLogger()
        if _root.handlers:
            _root.removeHandler(_root.handlers[0])
        _root.addHandler(self._stream_handler)

    @staticmethod
    def rotating_file_handler(file, log_level, formatter, max_bytes=104857600, backup_count=5):
        """
        Хендлер для вывода в файл
        :param file: файл лога
        :param log_level: уровень логирования
        :param formatter: формат лога
        :param max_bytes: максимальный размер файла в байтах
        :param backup_count: максимальное количество хранимых файлов логов
        :return: file-хендлер
        """
        h = logging.handlers.RotatingFileHandler(file, maxBytes=max_bytes, backupCount=backup_count)
        h.setLevel(log_level)
        h.setFormatter(formatter)
        return h

    @staticmethod
    def stream_handler(log_level, formatter):
        """
        Хендлер для вывода stdout
        :param log_level: уровень логирования
        :param formatter: форматирование
        :return: stream-хендлер
        """
        h = logging.StreamHandler(sys.stdout)
        h.setLevel(log_level)
        h.setFormatter(formatter)
        return h

    def __call__(self, name=None, *args, **kwargs):
        """
        Получить логер
        :param name: имя логера
        :return: логер
        """
        logger = logging.getLogger(name)
        if name:
            logger.setLevel(self.log_level)
            logger.addHandler(self._file_handler)
        return logger


get_logger = Logger()
