import json
import os

import rootpath
import xmltodict
from attributedict.collections import AttributeDict
from dateutil.parser import parse

DETECT_ROOT_PATTERN = '^.git|requirements.txt|setup.py|config$'


def root_dir():
    """
    Вернуть корень проекта
    """
    root_path = rootpath.detect(os.path.abspath(os.getcwd()), DETECT_ROOT_PATTERN)
    root_path = root_path or os.getcwd()
    return root_path


def xml_formatter(_data):
    """
    XML в словарь
    :param _data: xml-документ
    :return:
    """
    namespaces = {
        'http://www.w3.org/2003/05/soap-envelope': None,
        'http://schemas.compassplus.com/two/1.0/fimi.xsd': None,
        'http://schemas.compassplus.com/two/1.0/fimi_types.xsd': None,
        'http://schemas.compassplus.com/two/1.0/fimi_userdefined.xsd': None,
        'http://schemas.xmlsoap.org/soap/envelope/': None,
        'http://LetoBank/ServiceBroker/Services': None,
        'http://www.w3.org/2005/08/addressing': None,
        'http://schemas.xmlsoap.org/wsdl/': None,
        'http://siebel.com/CustomUI': None,
        'http://www.siebel.com/xml/ATC%20DBO%20ClientInfoById': None,
        'http://ekassir.com/eKassir/PaySystem/Server/eKassirV3Protocol': None,
        'http://www.w3.org/2001/XMLSchema-instance': None,
        'http://www.w3.org/2001/XMLSchema': None,
    }
    try:
        resp = xmltodict.parse(_data, process_namespaces=True, namespaces=namespaces)
        return json.loads(json.dumps(resp), encoding='utf8')
    except Exception as e:
        return _data


def find_item(obj, key):
    """
    рекурсивный поиск ключа по словарю
    :param obj: словарь
    :param key: ключ
    :return: значение
    """
    if key in obj:
        return obj[key]
    for k, v in obj.items():
        if isinstance(v, dict):
            item = find_item(v, key)
            if item is not None:
                return item


def str_to_datetime(value, **kwargs):
    """
    Строка даты и/или времени в объект DateTime
    :param value: строка
    :param kwargs: параметры парсинга (необязательно)
    :return: объект DateTime
    """
    try:
        value = parse(value, **kwargs)
    except (ValueError, AttributeError, OverflowError, TypeError):
        pass
    return value


def datetime_parser(value, current_key='', date_time_keys=('date', 'time', ), **parse_params):
    """
    Рекурсивное преобразование значений в словаре в объекты DateTime
    по указанным ключам или частям ключей date_time_keys
    :param value: словарь
    :param current_key: текущий ключ
    :param date_time_keys: список ключей/частей, в которых могут содержаться строки даты/времени
    :param parse_params: параметры парсинга (необязательно)
    :return:
    """
    if isinstance(value, dict):
        for k, v in value.items():
            value[k] = datetime_parser(v, k)
    elif isinstance(value, list):
        for index, row in enumerate(value):
            value[index] = datetime_parser(row)
    elif isinstance(value, str) and value:
        if list(filter(lambda x: x in current_key.lower(), date_time_keys)):
            value = str_to_datetime(value, **parse_params)
    return value


def validate_type(v, *allow_types):
    """
    Валидация объекта по типу
    :param v: объект
    :param allow_types: список разрешенных типов
    """
    allow_types = [type(t) if t is None else t for t in allow_types]
    if not isinstance(v, tuple(allow_types)):
        raise TypeError('Тип объекта: {}. Разрешенные типы: {}.'.format(type(v).__name__, allow_types))


def to_attribute_dict(structure):
    """
    Словарь в объект AttributeDict
    :param structure: словарь
    :return: AttributeDict
    """
    if isinstance(structure, dict):
        for key, value in list(structure.items()):
            if key == 'items':
                key = 'elements'
            structure[key] = to_attribute_dict(value)
        if 'items' in structure:
            del structure['items']
        if len(structure) == 1 and isinstance(list(structure.values())[0], list):
            return list(structure.values())[0]
        structure = AttributeDict(structure)
    elif isinstance(structure, list):
        for index, row in enumerate(structure):
            structure[index] = to_attribute_dict(row)
    return structure
